/* tslint:disable */
/* eslint-disable */
/**
 * elkar-app
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

import { mapValues } from '../runtime';
/**
 * 
 * @export
 * @interface PaginatedOutputTaskEventOutputRecordsInner
 */
export interface PaginatedOutputTaskEventOutputRecordsInner {
    /**
     * 
     * @type {Date}
     * @memberof PaginatedOutputTaskEventOutputRecordsInner
     */
    createdAt: Date;
    /**
     * 
     * @type {any}
     * @memberof PaginatedOutputTaskEventOutputRecordsInner
     */
    eventData: any | null;
    /**
     * 
     * @type {string}
     * @memberof PaginatedOutputTaskEventOutputRecordsInner
     */
    id: string;
    /**
     * 
     * @type {string}
     * @memberof PaginatedOutputTaskEventOutputRecordsInner
     */
    taskId: string;
    /**
     * 
     * @type {Date}
     * @memberof PaginatedOutputTaskEventOutputRecordsInner
     */
    updatedAt: Date;
}

/**
 * Check if a given object implements the PaginatedOutputTaskEventOutputRecordsInner interface.
 */
export function instanceOfPaginatedOutputTaskEventOutputRecordsInner(value: object): value is PaginatedOutputTaskEventOutputRecordsInner {
    if (!('createdAt' in value) || value['createdAt'] === undefined) return false;
    if (!('eventData' in value) || value['eventData'] === undefined) return false;
    if (!('id' in value) || value['id'] === undefined) return false;
    if (!('taskId' in value) || value['taskId'] === undefined) return false;
    if (!('updatedAt' in value) || value['updatedAt'] === undefined) return false;
    return true;
}

export function PaginatedOutputTaskEventOutputRecordsInnerFromJSON(json: any): PaginatedOutputTaskEventOutputRecordsInner {
    return PaginatedOutputTaskEventOutputRecordsInnerFromJSONTyped(json, false);
}

export function PaginatedOutputTaskEventOutputRecordsInnerFromJSONTyped(json: any, ignoreDiscriminator: boolean): PaginatedOutputTaskEventOutputRecordsInner {
    if (json == null) {
        return json;
    }
    return {
        
        'createdAt': (new Date(json['created_at'])),
        'eventData': json['event_data'],
        'id': json['id'],
        'taskId': json['task_id'],
        'updatedAt': (new Date(json['updated_at'])),
    };
}

export function PaginatedOutputTaskEventOutputRecordsInnerToJSON(json: any): PaginatedOutputTaskEventOutputRecordsInner {
    return PaginatedOutputTaskEventOutputRecordsInnerToJSONTyped(json, false);
}

export function PaginatedOutputTaskEventOutputRecordsInnerToJSONTyped(value?: PaginatedOutputTaskEventOutputRecordsInner | null, ignoreDiscriminator: boolean = false): any {
    if (value == null) {
        return value;
    }

    return {
        
        'created_at': ((value['createdAt']).toISOString()),
        'event_data': value['eventData'],
        'id': value['id'],
        'task_id': value['taskId'],
        'updated_at': ((value['updatedAt']).toISOString()),
    };
}

