-- This file should undo anything in `up.sql`
DROP TABLE IF EXISTS task_event_subscription;

DROP TABLE IF EXISTS task_event;

DROP TABLE IF EXISTS task_subscription;

DROP TYPE IF EXISTS task_event_subscription_status;

