/* tslint:disable */
/* eslint-disable */
/**
 * elkar-app
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

import { mapValues } from '../runtime';
import type { PushNotificationConfig } from './PushNotificationConfig';
import {
    PushNotificationConfigFromJSON,
    PushNotificationConfigFromJSONTyped,
    PushNotificationConfigToJSON,
    PushNotificationConfigToJSONTyped,
} from './PushNotificationConfig';

/**
 * Parameters for configuring push notifications
 * @export
 * @interface TaskPushNotificationConfig
 */
export interface TaskPushNotificationConfig {
    /**
     * 
     * @type {string}
     * @memberof TaskPushNotificationConfig
     */
    id: string;
    /**
     * 
     * @type {PushNotificationConfig}
     * @memberof TaskPushNotificationConfig
     */
    pushNotificationConfig: PushNotificationConfig;
}

/**
 * Check if a given object implements the TaskPushNotificationConfig interface.
 */
export function instanceOfTaskPushNotificationConfig(value: object): value is TaskPushNotificationConfig {
    if (!('id' in value) || value['id'] === undefined) return false;
    if (!('pushNotificationConfig' in value) || value['pushNotificationConfig'] === undefined) return false;
    return true;
}

export function TaskPushNotificationConfigFromJSON(json: any): TaskPushNotificationConfig {
    return TaskPushNotificationConfigFromJSONTyped(json, false);
}

export function TaskPushNotificationConfigFromJSONTyped(json: any, ignoreDiscriminator: boolean): TaskPushNotificationConfig {
    if (json == null) {
        return json;
    }
    return {
        
        'id': json['id'],
        'pushNotificationConfig': PushNotificationConfigFromJSON(json['push_notification_config']),
    };
}

export function TaskPushNotificationConfigToJSON(json: any): TaskPushNotificationConfig {
    return TaskPushNotificationConfigToJSONTyped(json, false);
}

export function TaskPushNotificationConfigToJSONTyped(value?: TaskPushNotificationConfig | null, ignoreDiscriminator: boolean = false): any {
    if (value == null) {
        return value;
    }

    return {
        
        'id': value['id'],
        'push_notification_config': PushNotificationConfigToJSON(value['pushNotificationConfig']),
    };
}

