/* tslint:disable */
/* eslint-disable */
/**
 * elkar-app
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

import { mapValues } from '../runtime';
import type { UnpaginatedOutputTenantOutputRecordsInner } from './UnpaginatedOutputTenantOutputRecordsInner';
import {
    UnpaginatedOutputTenantOutputRecordsInnerFromJSON,
    UnpaginatedOutputTenantOutputRecordsInnerFromJSONTyped,
    UnpaginatedOutputTenantOutputRecordsInnerToJSON,
    UnpaginatedOutputTenantOutputRecordsInnerToJSONTyped,
} from './UnpaginatedOutputTenantOutputRecordsInner';

/**
 * 
 * @export
 * @interface UnpaginatedOutputTenantOutput
 */
export interface UnpaginatedOutputTenantOutput {
    /**
     * 
     * @type {Array<UnpaginatedOutputTenantOutputRecordsInner>}
     * @memberof UnpaginatedOutputTenantOutput
     */
    records: Array<UnpaginatedOutputTenantOutputRecordsInner>;
}

/**
 * Check if a given object implements the UnpaginatedOutputTenantOutput interface.
 */
export function instanceOfUnpaginatedOutputTenantOutput(value: object): value is UnpaginatedOutputTenantOutput {
    if (!('records' in value) || value['records'] === undefined) return false;
    return true;
}

export function UnpaginatedOutputTenantOutputFromJSON(json: any): UnpaginatedOutputTenantOutput {
    return UnpaginatedOutputTenantOutputFromJSONTyped(json, false);
}

export function UnpaginatedOutputTenantOutputFromJSONTyped(json: any, ignoreDiscriminator: boolean): UnpaginatedOutputTenantOutput {
    if (json == null) {
        return json;
    }
    return {
        
        'records': ((json['records'] as Array<any>).map(UnpaginatedOutputTenantOutputRecordsInnerFromJSON)),
    };
}

export function UnpaginatedOutputTenantOutputToJSON(json: any): UnpaginatedOutputTenantOutput {
    return UnpaginatedOutputTenantOutputToJSONTyped(json, false);
}

export function UnpaginatedOutputTenantOutputToJSONTyped(value?: UnpaginatedOutputTenantOutput | null, ignoreDiscriminator: boolean = false): any {
    if (value == null) {
        return value;
    }

    return {
        
        'records': ((value['records'] as Array<any>).map(UnpaginatedOutputTenantOutputRecordsInnerToJSON)),
    };
}

