/* tslint:disable */
/* eslint-disable */
/**
 * elkar-app
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

import { mapValues } from '../runtime';
import type { ApplicationUserStatus } from './ApplicationUserStatus';
import {
    ApplicationUserStatusFromJSON,
    ApplicationUserStatusFromJSONTyped,
    ApplicationUserStatusToJSON,
    ApplicationUserStatusToJSONTyped,
} from './ApplicationUserStatus';

/**
 * 
 * @export
 * @interface ApplicationUserOutput
 */
export interface ApplicationUserOutput {
    /**
     * 
     * @type {string}
     * @memberof ApplicationUserOutput
     */
    email: string;
    /**
     * 
     * @type {string}
     * @memberof ApplicationUserOutput
     */
    firstName?: string | null;
    /**
     * 
     * @type {string}
     * @memberof ApplicationUserOutput
     */
    id: string;
    /**
     * 
     * @type {string}
     * @memberof ApplicationUserOutput
     */
    lastName?: string | null;
    /**
     * 
     * @type {ApplicationUserStatus}
     * @memberof ApplicationUserOutput
     */
    status: ApplicationUserStatus;
}



/**
 * Check if a given object implements the ApplicationUserOutput interface.
 */
export function instanceOfApplicationUserOutput(value: object): value is ApplicationUserOutput {
    if (!('email' in value) || value['email'] === undefined) return false;
    if (!('id' in value) || value['id'] === undefined) return false;
    if (!('status' in value) || value['status'] === undefined) return false;
    return true;
}

export function ApplicationUserOutputFromJSON(json: any): ApplicationUserOutput {
    return ApplicationUserOutputFromJSONTyped(json, false);
}

export function ApplicationUserOutputFromJSONTyped(json: any, ignoreDiscriminator: boolean): ApplicationUserOutput {
    if (json == null) {
        return json;
    }
    return {
        
        'email': json['email'],
        'firstName': json['first_name'] == null ? undefined : json['first_name'],
        'id': json['id'],
        'lastName': json['last_name'] == null ? undefined : json['last_name'],
        'status': ApplicationUserStatusFromJSON(json['status']),
    };
}

export function ApplicationUserOutputToJSON(json: any): ApplicationUserOutput {
    return ApplicationUserOutputToJSONTyped(json, false);
}

export function ApplicationUserOutputToJSONTyped(value?: ApplicationUserOutput | null, ignoreDiscriminator: boolean = false): any {
    if (value == null) {
        return value;
    }

    return {
        
        'email': value['email'],
        'first_name': value['firstName'],
        'id': value['id'],
        'last_name': value['lastName'],
        'status': ApplicationUserStatusToJSON(value['status']),
    };
}

