/* tslint:disable */
/* eslint-disable */
/**
 * elkar-app
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

import { mapValues } from '../runtime';
/**
 * 
 * @export
 * @interface PaginationOptions
 */
export interface PaginationOptions {
    /**
     * 
     * @type {number}
     * @memberof PaginationOptions
     */
    page?: number | null;
    /**
     * 
     * @type {number}
     * @memberof PaginationOptions
     */
    perPage?: number;
}

/**
 * Check if a given object implements the PaginationOptions interface.
 */
export function instanceOfPaginationOptions(value: object): value is PaginationOptions {
    return true;
}

export function PaginationOptionsFromJSON(json: any): PaginationOptions {
    return PaginationOptionsFromJSONTyped(json, false);
}

export function PaginationOptionsFromJSONTyped(json: any, ignoreDiscriminator: boolean): PaginationOptions {
    if (json == null) {
        return json;
    }
    return {
        
        'page': json['page'] == null ? undefined : json['page'],
        'perPage': json['per_page'] == null ? undefined : json['per_page'],
    };
}

export function PaginationOptionsToJSON(json: any): PaginationOptions {
    return PaginationOptionsToJSONTyped(json, false);
}

export function PaginationOptionsToJSONTyped(value?: PaginationOptions | null, ignoreDiscriminator: boolean = false): any {
    if (value == null) {
        return value;
    }

    return {
        
        'page': value['page'],
        'per_page': value['perPage'],
    };
}

