/* tslint:disable */
/* eslint-disable */
/**
 * elkar-app
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

import { mapValues } from '../runtime';
/**
 * 
 * @export
 * @interface ListApiKeysInput
 */
export interface ListApiKeysInput {
    /**
     * 
     * @type {Array<string>}
     * @memberof ListApiKeysInput
     */
    agentIdIn?: Array<string> | null;
}

/**
 * Check if a given object implements the ListApiKeysInput interface.
 */
export function instanceOfListApiKeysInput(value: object): value is ListApiKeysInput {
    return true;
}

export function ListApiKeysInputFromJSON(json: any): ListApiKeysInput {
    return ListApiKeysInputFromJSONTyped(json, false);
}

export function ListApiKeysInputFromJSONTyped(json: any, ignoreDiscriminator: boolean): ListApiKeysInput {
    if (json == null) {
        return json;
    }
    return {
        
        'agentIdIn': json['agent_id_in'] == null ? undefined : json['agent_id_in'],
    };
}

export function ListApiKeysInputToJSON(json: any): ListApiKeysInput {
    return ListApiKeysInputToJSONTyped(json, false);
}

export function ListApiKeysInputToJSONTyped(value?: ListApiKeysInput | null, ignoreDiscriminator: boolean = false): any {
    if (value == null) {
        return value;
    }

    return {
        
        'agent_id_in': value['agentIdIn'],
    };
}

