/* tslint:disable */
/* eslint-disable */
/**
 * elkar-app
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */


/**
 * 
 * @export
 */
export const TaskEventOrderBy = {
    CreatedAt: 'created-at',
    UpdatedAt: 'updated-at'
} as const;
export type TaskEventOrderBy = typeof TaskEventOrderBy[keyof typeof TaskEventOrderBy];


export function instanceOfTaskEventOrderBy(value: any): boolean {
    for (const key in TaskEventOrderBy) {
        if (Object.prototype.hasOwnProperty.call(TaskEventOrderBy, key)) {
            if (TaskEventOrderBy[key as keyof typeof TaskEventOrderBy] === value) {
                return true;
            }
        }
    }
    return false;
}

export function TaskEventOrderByFromJSON(json: any): TaskEventOrderBy {
    return TaskEventOrderByFromJSONTyped(json, false);
}

export function TaskEventOrderByFromJSONTyped(json: any, ignoreDiscriminator: boolean): TaskEventOrderBy {
    return json as TaskEventOrderBy;
}

export function TaskEventOrderByToJSON(value?: TaskEventOrderBy | null): any {
    return value as any;
}

export function TaskEventOrderByToJSONTyped(value: any, ignoreDiscriminator: boolean): TaskEventOrderBy {
    return value as TaskEventOrderBy;
}

