/* tslint:disable */
/* eslint-disable */
/**
 * elkar-app
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

import { mapValues } from '../runtime';
/**
 * 
 * @export
 * @interface PartTypeOneOf
 */
export interface PartTypeOneOf {
    /**
     * 
     * @type {string}
     * @memberof PartTypeOneOf
     */
    text: string;
    /**
     * 
     * @type {string}
     * @memberof PartTypeOneOf
     */
    type: PartTypeOneOfTypeEnum;
}


/**
 * @export
 */
export const PartTypeOneOfTypeEnum = {
    Text: 'text'
} as const;
export type PartTypeOneOfTypeEnum = typeof PartTypeOneOfTypeEnum[keyof typeof PartTypeOneOfTypeEnum];


/**
 * Check if a given object implements the PartTypeOneOf interface.
 */
export function instanceOfPartTypeOneOf(value: object): value is PartTypeOneOf {
    if (!('text' in value) || value['text'] === undefined) return false;
    if (!('type' in value) || value['type'] === undefined) return false;
    return true;
}

export function PartTypeOneOfFromJSON(json: any): PartTypeOneOf {
    return PartTypeOneOfFromJSONTyped(json, false);
}

export function PartTypeOneOfFromJSONTyped(json: any, ignoreDiscriminator: boolean): PartTypeOneOf {
    if (json == null) {
        return json;
    }
    return {
        
        'text': json['text'],
        'type': json['type'],
    };
}

export function PartTypeOneOfToJSON(json: any): PartTypeOneOf {
    return PartTypeOneOfToJSONTyped(json, false);
}

export function PartTypeOneOfToJSONTyped(value?: PartTypeOneOf | null, ignoreDiscriminator: boolean = false): any {
    if (value == null) {
        return value;
    }

    return {
        
        'text': value['text'],
        'type': value['type'],
    };
}

