/* tslint:disable */
/* eslint-disable */
/**
 * elkar-app
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

import { mapValues } from '../runtime';
import type { AgentOutput } from './AgentOutput';
import {
    AgentOutputFromJSON,
    AgentOutputFromJSONTyped,
    AgentOutputToJSON,
    AgentOutputToJSONTyped,
} from './AgentOutput';

/**
 * 
 * @export
 * @interface RetrieveAgentsOutput
 */
export interface RetrieveAgentsOutput {
    /**
     * 
     * @type {Array<AgentOutput>}
     * @memberof RetrieveAgentsOutput
     */
    agents: Array<AgentOutput>;
}

/**
 * Check if a given object implements the RetrieveAgentsOutput interface.
 */
export function instanceOfRetrieveAgentsOutput(value: object): value is RetrieveAgentsOutput {
    if (!('agents' in value) || value['agents'] === undefined) return false;
    return true;
}

export function RetrieveAgentsOutputFromJSON(json: any): RetrieveAgentsOutput {
    return RetrieveAgentsOutputFromJSONTyped(json, false);
}

export function RetrieveAgentsOutputFromJSONTyped(json: any, ignoreDiscriminator: boolean): RetrieveAgentsOutput {
    if (json == null) {
        return json;
    }
    return {
        
        'agents': ((json['agents'] as Array<any>).map(AgentOutputFromJSON)),
    };
}

export function RetrieveAgentsOutputToJSON(json: any): RetrieveAgentsOutput {
    return RetrieveAgentsOutputToJSONTyped(json, false);
}

export function RetrieveAgentsOutputToJSONTyped(value?: RetrieveAgentsOutput | null, ignoreDiscriminator: boolean = false): any {
    if (value == null) {
        return value;
    }

    return {
        
        'agents': ((value['agents'] as Array<any>).map(AgentOutputToJSON)),
    };
}

