/* tslint:disable */
/* eslint-disable */
/**
 * elkar-app
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

import { mapValues } from '../runtime';
/**
 * 
 * @export
 * @interface IsRegisteredOutput
 */
export interface IsRegisteredOutput {
    /**
     * 
     * @type {boolean}
     * @memberof IsRegisteredOutput
     */
    isOnTenant?: boolean | null;
    /**
     * 
     * @type {boolean}
     * @memberof IsRegisteredOutput
     */
    isRegistered: boolean;
    /**
     * 
     * @type {boolean}
     * @memberof IsRegisteredOutput
     */
    needToCreateTenant?: boolean | null;
}

/**
 * Check if a given object implements the IsRegisteredOutput interface.
 */
export function instanceOfIsRegisteredOutput(value: object): value is IsRegisteredOutput {
    if (!('isRegistered' in value) || value['isRegistered'] === undefined) return false;
    return true;
}

export function IsRegisteredOutputFromJSON(json: any): IsRegisteredOutput {
    return IsRegisteredOutputFromJSONTyped(json, false);
}

export function IsRegisteredOutputFromJSONTyped(json: any, ignoreDiscriminator: boolean): IsRegisteredOutput {
    if (json == null) {
        return json;
    }
    return {
        
        'isOnTenant': json['is_on_tenant'] == null ? undefined : json['is_on_tenant'],
        'isRegistered': json['is_registered'],
        'needToCreateTenant': json['need_to_create_tenant'] == null ? undefined : json['need_to_create_tenant'],
    };
}

export function IsRegisteredOutputToJSON(json: any): IsRegisteredOutput {
    return IsRegisteredOutputToJSONTyped(json, false);
}

export function IsRegisteredOutputToJSONTyped(value?: IsRegisteredOutput | null, ignoreDiscriminator: boolean = false): any {
    if (value == null) {
        return value;
    }

    return {
        
        'is_on_tenant': value['isOnTenant'],
        'is_registered': value['isRegistered'],
        'need_to_create_tenant': value['needToCreateTenant'],
    };
}

