/* tslint:disable */
/* eslint-disable */
/**
 * elkar-app
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

import { mapValues } from '../runtime';
import type { TaskEventOrderBy } from './TaskEventOrderBy';
import {
    TaskEventOrderByFromJSON,
    TaskEventOrderByFromJSONTyped,
    TaskEventOrderByToJSON,
    TaskEventOrderByToJSONTyped,
} from './TaskEventOrderBy';

/**
 * 
 * @export
 * @interface GetTaskEventsQuery
 */
export interface GetTaskEventsQuery {
    /**
     * 
     * @type {Array<string>}
     * @memberof GetTaskEventsQuery
     */
    idIn?: Array<string> | null;
    /**
     * 
     * @type {number}
     * @memberof GetTaskEventsQuery
     */
    limit?: number | null;
    /**
     * 
     * @type {TaskEventOrderBy}
     * @memberof GetTaskEventsQuery
     */
    orderBy?: TaskEventOrderBy | null;
    /**
     * 
     * @type {number}
     * @memberof GetTaskEventsQuery
     */
    page?: number | null;
    /**
     * 
     * @type {Array<string>}
     * @memberof GetTaskEventsQuery
     */
    taskIdIn?: Array<string> | null;
}



/**
 * Check if a given object implements the GetTaskEventsQuery interface.
 */
export function instanceOfGetTaskEventsQuery(value: object): value is GetTaskEventsQuery {
    return true;
}

export function GetTaskEventsQueryFromJSON(json: any): GetTaskEventsQuery {
    return GetTaskEventsQueryFromJSONTyped(json, false);
}

export function GetTaskEventsQueryFromJSONTyped(json: any, ignoreDiscriminator: boolean): GetTaskEventsQuery {
    if (json == null) {
        return json;
    }
    return {
        
        'idIn': json['id_in'] == null ? undefined : json['id_in'],
        'limit': json['limit'] == null ? undefined : json['limit'],
        'orderBy': json['order_by'] == null ? undefined : TaskEventOrderByFromJSON(json['order_by']),
        'page': json['page'] == null ? undefined : json['page'],
        'taskIdIn': json['task_id_in'] == null ? undefined : json['task_id_in'],
    };
}

export function GetTaskEventsQueryToJSON(json: any): GetTaskEventsQuery {
    return GetTaskEventsQueryToJSONTyped(json, false);
}

export function GetTaskEventsQueryToJSONTyped(value?: GetTaskEventsQuery | null, ignoreDiscriminator: boolean = false): any {
    if (value == null) {
        return value;
    }

    return {
        
        'id_in': value['idIn'],
        'limit': value['limit'],
        'order_by': TaskEventOrderByToJSON(value['orderBy']),
        'page': value['page'],
        'task_id_in': value['taskIdIn'],
    };
}

